from pyba.utils.prompts.system_prompt import system_prompt as system_instruction
from pyba.utils.prompts.general_prompt import general_prompt
from pyba.utils.prompts.output_general_prompt import output_prompt
from pyba.utils.prompts.output_system_prompt import (
    output_system_prompt as output_system_instruction,
)
